unit DDatabase;

interface

uses
  System.SysUtils, System.Classes, Data.DB, Data.SqlExpr, Data.DBXMSSQL,
  System.Generics.Collections, Data.DBXCommon;

type
  TdmDatabase = class(TDataModule)
    MyConnection: TSQLConnection;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    FConnectionList: TDictionary<Integer, TSQLConnection>;
  class var
    FInstance: TdmDatabase;
    function GetConnectionList: TDictionary<Integer, TSQLConnection>;
    function GetHostName: string;
    function GetUser_Name: string;
    function GetPassword: string;
    function GetSQLConnection: TSQLConnection;
    procedure SetHostName(const Value: string);
    procedure SetUser_Name(const Value: string);
    procedure SetPassword(const Value: string);
  protected
  public
    class constructor Create;
    class destructor Destroy;
    function CloneConnection: TSQLConnection;
    class function Instance: TdmDatabase;
    property ConnectionList: TDictionary<Integer, TSQLConnection> read
        GetConnectionList;
    property HostName: string read GetHostName write SetHostName;
    property User_Name: string read GetUser_Name write SetUser_Name;
    property Password: string read GetPassword write SetPassword;
    property SQLConnection: TSQLConnection read GetSQLConnection;
  end;

implementation

{%CLASSGROUP 'System.Classes.TPersistent'}

{$R *.dfm}

uses
//  CodeSiteLogging,
  MidasLib, Winapi.Windows, System.IniFiles, System.IOUtils;

class constructor TdmDatabase.Create;
begin
  FInstance := TdmDatabase.Create(nil);
end;

class destructor TdmDatabase.Destroy;
begin
  FInstance.Free;
end;

function TdmDatabase.CloneConnection: TSQLConnection;
begin
  result := TSQLConnection.Create(nil);
  result.LoadParamsOnConnect := false;
  result.DriverName := MyConnection.DriverName;
  result.GetDriverFunc := MyConnection.GetDriverFunc;
  result.LibraryName := MyConnection.LibraryName;
  result.VendorLib := MyConnection.VendorLib;
  result.LoginPrompt := MyConnection.LoginPrompt;
  result.Params := MyConnection.Params;
  result.AfterConnect := MyConnection.AfterConnect;
  result.AfterDisconnect := MyConnection.AfterDisconnect;
  result.KeepConnection := false;
end;

procedure TdmDatabase.DataModuleCreate(Sender: TObject);
var
  ini: TMemInifile;
  iniFileName: string;
begin
  MyConnection.Connected := false;
//  CodeSiteManager.ConnectUsingTcp;
  iniFileName := ChangeFileExt(ParamStr(0), '.ini');
  if TFile.Exists(iniFileName) then begin
    MyConnection.Params.Values['OSAuthentication'] := 'False';
    MyConnection.Params.Values['OS Authentication'] := 'False';
    ini := TMemIniFile.Create(iniFileName);
    try
      HostName := ini.ReadString('SQLServer', 'HostName', HostName);
      User_Name := ini.ReadString('SQLServer', 'User_Name', User_Name);
      Password := ini.ReadString('SQLServer', 'Password', Password);
    finally
      ini.Free;
    end;
  end;
  MyConnection.Params.Values['Mars_Connection'] := 'True';
end;

procedure TdmDatabase.DataModuleDestroy(Sender: TObject);
begin
  FConnectionList.Free;
end;

function TdmDatabase.GetConnectionList: TDictionary<Integer, TSQLConnection>;
begin
  if FConnectionList = nil then
    FConnectionList := TDictionary<Integer, TSQLConnection>.Create;
  result := FConnectionList;
end;

function TdmDatabase.GetHostName: string;
begin
  Result := MyConnection.Params.Values['HostName'];
end;

function TdmDatabase.GetUser_Name: string;
begin
  Result := MyConnection.Params.Values['User_Name'];
end;

function TdmDatabase.GetPassword: string;
begin
  Result := MyConnection.Params.Values['Password'];
end;

function TdmDatabase.GetSQLConnection: TSQLConnection;
var
  threadID: NativeInt;
begin
  threadID := GetCurrentThreadID;
  if ConnectionList.ContainsKey(threadID) then begin
    result := ConnectionList[threadID];
  end
  else begin
    result := CloneConnection;
    ConnectionList.Add(threadID, result);
  end;
end;

class function TdmDatabase.Instance: TdmDatabase;
begin
  if FInstance = nil  then
    FInstance := TdmDatabase.Create(nil);
  Result := FInstance;
end;

procedure TdmDatabase.SetHostName(const Value: string);
begin
  MyConnection.Params.Values['HostName'] := Value;
end;

procedure TdmDatabase.SetUser_Name(const Value: string);
begin
  MyConnection.Params.Values['User_Name'] := Value;
end;

procedure TdmDatabase.SetPassword(const Value: string);
begin
  MyConnection.Params.Values['Password'] := Value;
end;

end.
